# -*- coding: utf-8 -*-
"""
:Package:           salespyforce.errors
:Synopsis:          This module includes custom exceptions and handlers
:Created By:        Jeff Shurtliff
:Last Modified:     Jeff Shurtliff
:Modified Date:     13 Mar 2023
"""

__all__ = ['exceptions', 'handlers']

from . import exceptions, handlers
